﻿--Bar creation module

local bars = {}
local tsort = table.sort

function SN:CreateBar(frame, cooldown, duration, caster, guid, spell)

	--Make our lives easier with some variables
	local v = SN.Media
	local class, classFileName, classID = UnitClass(caster)
	local bar

	--Start the CD Timer
	bar = SN.timers:New(v.media.minTex, frame:GetWidth(), frame.height)
	bar:SetParent(frame)
	--Check to see if ElvUI is on so we can make the frames match
	if (IsAddOnLoaded("ElvUI")) then
		SN:SNCreateBG(bar)
	else
	--Standard Backdrop settings
		bar:SetAlpha(1)
		bar:SetBackdrop({
			bgFile = v.media.blankTex, 
			edgeFile = v.media.blankTex, 
			tile = false, tileSize = 0, edgeSize = 2, 
			insets = { left = -2, right = 2, top = 2, bottom = -2 }
		})
		bar:SetBackdropColor(0, 0, 0, 0)
		bar:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
	end
	
	--Grab the player's class color and the icon of the spell
	local colortmp = RAID_CLASS_COLORS[classFileName]
	color = { colortmp.r, colortmp.g, colortmp.b }
	local _, _, icon = GetSpellInfo(cooldown)
	
	--if icon == nil then
	--	icon = "Interface\\Icons\\Ability_Hunter_Readiness"
	--end
	bar:SetIcon(icon)
	
	bar:SetColor(unpack(color))
	bar:SetDuration(duration)
	bar.candyBarDuration:SetFont(v.media.font, 16)
	bar.candyBarLabel:SetFont(v.media.font, 16)
	bar.candyBarLabel:SetJustifyH("LEFT")

	--Setup the data keys that the bar will hold
	bar:Set("SN:arg1", frame)
	bar:Set("SN:arg2", cooldown)
	bar:Set("SN:arg3", guid)
	bar:Set("SN:arg4", caster)
	bar:Set("SN:arg5", spell)
	bar:Set("SN:arg6", classID)
	bar:Set("SN:arg7", duration)
	bar.bartype = { cooldown, guid, caster }
	bar.TimeSinceLastUpdate = 0
	
	--Make sure only the CD monitor gets the caster and spell names
	if frame == SN.F.debuffs then
		bar:SetLabel(spell)
	elseif frame == SN.F.cooldowns then
		bar:SetLabel(caster.."       "..spell)
	end
	
	--Add some things the bar should do on the events
	bar:SetScript("OnEvent", function(self, event, ...)
		SN:EventHandler(self, event, ...)
	end)
	--Add bar to the appropriate group
	bar.frame = frame

	if frame == SN.F.debuffs then
		DBVisible[bar] = true
	elseif frame == SN.F.cooldowns then
		CDVisible[bar] = true
	end

	SN:Sort(frame)
	LibStub("LibCandyBar-3.0").RegisterCallback(bar, "LibCandyBar_Stop", barRemove)
	bar:Start()

	--So we can check on our combat changes
	bar:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
end

function barRemove(callback, self)
	if self.frame == SN.F.debuffs then
		if DBVisible[self] then
			DBVisible[self] = nil
		end
	elseif self.frame == SN.F.cooldowns then
		if CDVisible[self] then
			CDVisible[self] = nil
		end
	end
	SN:Sort(self.frame)
end

function SN:SortBars(a, b, field)
	-- sort using the time remaining
	if field == "time" then
		return (a.remaining < b.remaining)
	-- sort using the bar's label
	elseif field == "label" then
		return (a.candyBarLabel < b.candyBarLabel)
	-- sort using a data field
	else
		return (a:Get(field) < b:Get(field))
	end
end

local tmp = {}

function SN:Sort(frame)

	if frame == SN.F.debuffs then
		wipe(tmp)
		for bar in pairs(DBVisible) do
			tmp[#tmp + 1] = bar
		end
		table.sort(tmp, function(a, b) return SN:SortBars(a, b, frame.sortfield) end)
		local lastBar = nil
		local startpoint = frame.startpoint
		for i, bar in next, tmp do
			bar:ClearAllPoints()
			if not lastBar then
				if startpoint == "topleft" then
					bar:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", 0, frame.spacing_v)
				elseif startpoint == "topright" then
					bar:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", 0, frame.spacing_v)
				elseif startpoint == "bottomright" then
					bar:SetPoint("TOPRIGHT", frame, "BOTTOMRIGHT", 0, -frame.spacing_v)
				else
					bar:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 0, -frame.spacing_v)
				end
			else
				if startpoint == "topleft" then
					bar:SetPoint("BOTTOMLEFT", lastBar, "TOPLEFT", 0, frame.spacing_v)
				elseif startpoint == "topright" then
					bar:SetPoint("BOTTOMRIGHT", lastBar, "TOPRIGHT", 0, frame.spacing_v)
				elseif startpoint == "bottomright" then
					bar:SetPoint("TOPRIGHT", lastBar, "BOTTOMRIGHT", 0, -frame.spacing_v)
				else
					bar:SetPoint("TOPLEFT", lastBar, "BOTTOMLEFT", 0, -frame.spacing_v)
				end
			end
			lastBar = bar
			bar:Show()
		end
	elseif frame == SN.F.cooldowns then
		wipe(tmp)
		for bar in pairs(CDVisible) do
			tmp[#tmp + 1] = bar
		end
		table.sort(tmp, function(a, b) return SN:SortBars(a, b, frame.sortfield) end)
		local lastBar = nil
		local startpoint = frame.startpoint
		for i, bar in next, tmp do
			bar:ClearAllPoints()
			if not lastBar then
				if startpoint == "topleft" then
					bar:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", 0, frame.spacing_v)
				elseif startpoint == "topright" then
					bar:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", 0, frame.spacing_v)
				elseif startpoint == "bottomright" then
					bar:SetPoint("TOPRIGHT", frame, "BOTTOMRIGHT", 0, -frame.spacing_v)
				else
					bar:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 0, -frame.spacing_v)
				end
			else
				if startpoint == "topleft" then
					bar:SetPoint("BOTTOMLEFT", lastBar, "TOPLEFT", 0, frame.spacing_v)
				elseif startpoint == "topright" then
					bar:SetPoint("BOTTOMRIGHT", lastBar, "TOPRIGHT", 0, frame.spacing_v)
				elseif startpoint == "bottomright" then
					bar:SetPoint("TOPRIGHT", lastBar, "BOTTOMRIGHT", 0, -frame.spacing_v)
				else
					bar:SetPoint("TOPLEFT", lastBar, "BOTTOMLEFT", 0, -frame.spacing_v)
				end
			end
			lastBar = bar
			bar:Show()
		end
	end
end
	
function SN:Find(frame, key, value)
	for i, k in pairs(frame.bars) do
		if k.data[key] and k.data[key] == value then return k end
	end
	return nil
end

