﻿-- Config UI

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")

local selectedSpell
local cdtoggle = {}
local bloodlustId

local function profileUpdate()
	db = SN.db.profile
end

function SN:SetupOptions()

	--Register the Ace database
	local db = AceDB:New("SNDB", SN.V.defaults, true)
	SN.db = db

	SN.options.args.profile = AceDBOptions:GetOptionsTable(SN.db)
	
	AceConfig:RegisterOptionsTable("SN", SN.options, nil)
	
	SN.optionsFrames = {}
	SN.optionsFrames.general = AceConfigDialog:AddToBlizOptions("SN", "Serenity Now", nil, "general")
	SN.optionsFrames.profile = AceConfigDialog:AddToBlizOptions("SN", "Profiles", "Serenity Now", "profile")

	SN.db.RegisterCallback(self, "OnProfileChanged", profileUpdate)
	SN.db.RegisterCallback(self, "OnProfileCopied", profileUpdate)
	SN.db.RegisterCallback(self, "OnProfileReset", profileUpdate)

end

SN.TexCoords = {.08, .92, .08, .92}

local function SpellName(id)
	local name, _, _, _, _, _, _, _, _ = GetSpellInfo(id) 	
	if not name then
		return 'Null'
	else
		return name
	end
end

SN.V.defaults = {
	profile = {
		["debug"] = {
			enable = false,
		},
		["CD"] = {
			castannounce = true,
			show = "always",
			raid = true,
		},
		["dispel"] = {
			enable = true,
		},
		["threat"] = {
			enable = true,
		},
		["debuffs"] = {
			enable = true,
		},
		["Frames"] = {
			lock = true,
			["threat"] = { "CENTER", nil, "CENTER", 0, 100 },
			["debuffs"] = { "CENTER", nil, "CENTER", 350, -450 },
			["elvdebuffs"] = { "CENTER", nil, "CENTER", 0, -450	},
			["cooldowns"] = { "BOTTOMRIGHT", nil, "BOTTOMRIGHT", -100, 0 },
			["elvcooldowns"] = { "TOPLEFT", nil, "TOPLEFT", 5, 0 },
			["dispel"] = { "CENTER", nil, "CENTER", 0, -100 },
			["power"] = { "CENTER", nil, "CENTER", 0, 100 },
			["cc"] = { "CENTER", nil, "CENTER", 0, 0 },
		},
		["powerbar"] = {
			tickenable = false,
			enable = true,
			["hunter"] = {
				{	--Beast Mastery
					{ true, "Kill Command", true, false, {1,1,1,1}, 0 },
					{ false, "Glaive Toss", true, false, {1,1,1,1}, 0 },
				},
				{	--Marksman
					{ true, "Chimera Shot", true, false, {1,1,1,1}, 0 },
					{ false, "Aimed Shot", true, false, {1,1,1,1}, 0 },
				},
				{	--Survival
					{ true, "Explosive Shot", true, false, {1,1,1,1}, 0 },
					{ false, "Black Arrow", true, false, {1,1,1,1}, 0 },
				},
			},
			["rogue"] = {
				{	--Assassination
					{ true, "Mutilate", true, false, {1,1,1,1}, 0 },
					{ false, "Dispatch", true, false, {1,1,1,1}, 0 },
				},
				{	--Combat
					{ true, "Sinister Strike", true, false, {1,1,1,1}, 0 },
					{ true, "Eviscerate", true, false, {1,1,1,1}, 0 },
				},
				{	--Subtlety
					{ true, "Backstab", true, false, {1,1,1,1}, 0 },
					{ false, "Eviscerate", true, false, {1,1,1,1}, 0 },
				},
			},
			["druid"] = {
				{	--Feral
					{ true, "Shred", true, false, {1,1,1,1}, 0 },
					{ false, "Mangle", true, false, {1,1,1,1}, 0 },
				},
				{	--Guardian
					{ true, "Savage Defense", true, false, {1,1,1,1}, 0 },
					{ false, "Frenzied Regeneration", true, false, {1,1,1,1}, 0 },
				},
			},
			["warrior"] = {
				{	--Arms
					{ true, "Slam", true, false, {1,1,1,1}, 0 },
					{ false, "Execute", true, false, {1,1,1,1}, 0 },
				},
				{	--Fury
					{ true, "Heroic Strike", true, false, {1,1,1,1}, 0 },
					{ false, "Execute", true, false, {1,1,1,1}, 0 },
				},
				{	--Prot
					{ true, "Shield Block", true, false, {1,1,1,1}, 0 },
					{ false, "Shield Barrier", true, false, {1,1,1,1}, 0 },
				},
			},
		},
	},
}

SN.options =  {
	type = "group",
	name = "Serenity Now".." "..SN.ver,
	args = {
		general = {
			type = "group",
			name = "general",
			args = {
				descrip = {
					order = 0,
					type = "description",
					name = "A tool to have as much of the needed information from each class\nwithin eye sight so that you can say Serenity Now!",
				},
				h1 = {
                    order = 1,
                    type = "header",
                    name = "",
                },
				
				lock = {
					order = 2,
					name = "Lock Frames",
					type = 'toggle',
					get = function()
						return SN.db.profile.Frames.lock 
					end,
					set = function(key, value)
						--SN.db.profile.Frames.lock = value
						SN:MoveFrames(value)
					end,
				},
				debug = {
					order = 2,
					name = "Debug Mode",
					type = 'toggle',
					get = function()
						return SN.db.profile.debug.enable 
					end,
					set = function(key, value)
						SN.db.profile.debug.enable = value
					end,
				},
				configure = {
					order = 6,
					type = "execute",
					name = "Apply Changes",
					desc = "Apply changes reload the UI.",
					func = function()
						ReloadUI()
					end,
					order = 1,
					width = "normal",
				},
				h2 = {
                    order = 7,
                    type = "header",
                    name = "",
                },
				cooldowns = {
					order = 24,
					type = "group",
					name = "Cooldowns",
					args = {
						selectClass = {
							name = "Select Class",
							type = "select",
							order = 0,
							values = function()
								local values = { } 
								for _, class in pairs(CLASS_SORT_ORDER) do
									class1 = strsub(class, 1, 1)
									class1 = strconcat(class1, strlower(strsub(class, 2)))
									values[class] = class1
								end
								return values
							end,
							get = function(info) return selectedClass end,
							set = function(info, value)
								selectedClass = value
							end,
						},
						selectSpell = {
							name = "Select Spell",
							type = "select",
							order = 1,						
							values = function()
								local values = {}
								for _, spell in pairs(SN.db.profile.cooldowns) do
									if selectedClass == spell.class then
										local name = GetSpellInfo(spell.id)
										values[spell.id] = name
									end
								end
								return values
							end,
							get = function(info) return selectedSpell end,
							set = function(info, value)
								selectedSpell = value
							end,
						},
						enableSpell = {	
							name = "Enabled",
							type = 'toggle',
							order = 2,
							get = function(info) 
								for _, spell in pairs(SN.db.profile.cooldowns) do
									if selectedSpell == spell.id then
										return spell.enable
									end
								end
							end,
							set = function(info, value) 
								for _, spell in pairs(SN.V.defaults.profile.cooldowns) do
									if selectedSpell == spell.id then
										spell.enable = value
									end
								end
							end,
						},
						h2 = {
							order = 4,
							type = "header",
							name = "",
						},
						castannounce = {
							type = "toggle",
							name = "Announce Casts",
							order = 5,
							get = function()
								return SN.db.profile.CD.castannounce
							end,
							set = function(key, value)
								SN.db.profile.CD.castannounce = value
							end,
						},
						--raid = {
						--	order = 6,
						--	name = "Raid Only CDs",
						--	desc = "Shows the raid wide cooldowns instead of personal cooldowns. Must be the group leader for this feature to work.",
						--	type = "toggle",
						--	get = function()
						--		return SN.db.profile.CD.raid
						--	end,
						--	set = function(key, value)
						--		SN.db.profile.CD.raid = value
						--	end,
						--},
					},
				},
				debuffs = {
					order = 25,
					type = "group",
					name = "Debuffs",
					args = {
						dbenable = {
							type = "toggle",
							name = "Enable Debuff Bars",
							order = 0,
							get = function()
								return SN.db.profile.debuffs.enable
							end,
							set = function(key, value)
								SN.db.profile.debuffs.enable = value
							end,
						},
					},
				},
				--threat = {
				--	order = 26,
				--	type = "group",
				--	name = "Threat",
				--	args = { 
				--		threatenable = {
				--			type = "toggle",
				--			name = "Enable Threat System",
				--			order = 0,
				--			get = function()
				--				return SN.db.profile.threat.enable
				--			end,
				--			set = function(key, value)
				--				if value == true then
				--					SN.F.threat:Show()
				--				else
				--					SN.F.threat:Hide()
				--				end
				--				SN.db.profile.threat.enable = value
				--			end,
				--		},
				--	},
				--},
				dispel = {
					order = 24,
					type = "group",
					name = "Dispel",
					args = {
						dispelenable = {
							type = "toggle",
							name = "Enable Dispel System",
							order = 0,
							get = function()
								return SN.db.profile.dispel.enable
							end,
							set = function(key, value)
								if value == true then
									SN.F.dispel:Show()
								else
									SN.F.dispel:Hide()
								end
								SN.db.profile.dispel.enable = value
							end,
						},
					},
				},
				powerbar = {
					order = 24,
					type = "group",
					name = "Powerbar",
					args = {
						powenable = {
							order = 0,
							type = "toggle",
							name = "Enable Powerbar",
							get = function()
								return SN.db.profile.powerbar.enable
							end,
							set = function(key, value)
								if value == true then
									SN:UpdatePowerType()
								else
									SN.F.powercontainer:Hide()
								end
								SN.db.profile.powerbar.enable = value
							end,
						},
						powertick = {
							order = 1,
							name = "Powebar Ticks",
							desc = "Tick marks on the powerbar to represent how much power you need to cast your main spell.",
							type = "toggle",
							get = function()
								return SN.db.profile.powerbar.tickenable
							end,
							set = function(key, value)
								SN.db.profile.powerbar.tickenable = value
								SN:PowebarTicks()
							end,
						},
					},
				},
			},
		},
	}
}